{-# LANGUAGE OverloadedStrings #-}

module Main where

import PUBMED.Client
import PUBMED.Parser
import Network.HTTP.Client (newManager)
import Network.HTTP.Client.TLS (tlsManagerSettings)
import Servant.Client (runClientM, mkClientEnv, BaseUrl(..), Scheme(..))
import Text.XML (parseLBS_, def)
import Text.XML.Cursor (fromDocument, Cursor)

import qualified Data.ByteString.Lazy as LBS

runParser :: Show res => (Cursor -> res) -> LBS.ByteString -> res
runParser parser = parser . fromDocument . parseLBS_ def

main :: IO ()
main = do
  manager' <- newManager tlsManagerSettings
  res <- runClientM
    (search (Just "organ"))
    (mkClientEnv manager' $ BaseUrl Https "eutils.ncbi.nlm.nih.gov" 443 "entrez/eutils")
  case res of
    (Left err) -> print err
    (Right (BsXml docs)) -> do
      let docIds = runParser parseDocId docs
      res' <- runClientM
        (fetch (Just "pubmed") (Just "abstract") docIds)
        (mkClientEnv manager' $ BaseUrl Https "eutils.ncbi.nlm.nih.gov" 443 "entrez/eutils")
      case res' of
        (Left err) -> print err
        (Right (BsXml abstracts)) -> do
          pmeds <- pubMedParser abstracts
          print pmeds
